import 'source-map-support/register';
import { Counter, Event, Gauge, Histogram, Logger, Meter, Metric, MetricRegistry, MetricSetReportContext, MetricType, MonotoneCounter, OverallReportContext, ReportingResult, ScheduledMetricReporter, ScheduledMetricReporterOptions, Tags, Timer } from 'inspector-metrics';
/**
 * Interface for getting a certain information using the specified metric metadata -
 * e.g. name of the index, metric type, etc.
 */
export declare type MetricInfoDeterminator = (registry: MetricRegistry, metric: Metric, type: MetricType, date: Date) => string;
/**
 * Interface for building a document for a metric.
 */
export declare type MetricDocumentBuilder = (registry: MetricRegistry, metric: Metric, type: MetricType, date: Date, tags: Tags) => {};
/**
 * Options for {@link ElasticsearchMetricReporter}.
 *
 * @export
 * @interface ElasticsearchMetricReporterOption
 * @extends {ScheduledMetricReporterOptions}
 */
export interface ElasticsearchMetricReporterOption extends ScheduledMetricReporterOptions {
    /**
     * Elasticsearch client options.
     *
     * @memberof ElasticsearchMetricReporterOption
     */
    readonly clientOptions: {};
    /**
     * Logger instance used to report errors.
     *
     * @type {Logger}
     * @memberof ElasticsearchMetricReporterOption
     */
    log?: Logger;
    /**
     * Used to get the name of the index.
     *
     * @type {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporterOption
     */
    readonly indexnameDeterminator?: MetricInfoDeterminator;
    /**
     * Used to get the type of the metric instance.
     *
     * @type {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporterOption
     */
    readonly typeDeterminator?: MetricInfoDeterminator;
    /**
     * Used to build the document for a metric.
     *
     * @type {MetricDocumentBuilder}
     * @memberof ElasticsearchMetricReporterOption
     */
    readonly metricDocumentBuilder?: MetricDocumentBuilder;
}
/**
 * A MetricReporter extension used to publish metric values to elasticsearch.
 *
 * @export
 * @class ElasticsearchMetricReporter
 * @extends {MetricReporter}
 */
export declare class ElasticsearchMetricReporter extends ScheduledMetricReporter<ElasticsearchMetricReporterOption, Array<{}>> {
    /**
     * Returns a {@link MetricInfoDeterminator} that returns 'metric' as type.
     *
     * @static
     * @returns {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporter
     */
    static defaultTypeDeterminator(): MetricInfoDeterminator;
    /**
     * Returns a {@link MetricInfoDeterminator} that returns an indexname like '<baseName>-yyyy-mm-dd'.
     *
     * @static
     * @param {string} baseName The
     * @returns {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporter
     */
    static dailyIndex(baseName: string): MetricInfoDeterminator;
    /**
     * Returns a {@link MetricDocumentBuilder} that builds an object for a metric like this:
     *
     * {
     *
     *  name: ..., // name of metric
     *
     *  group: ..., // group of metric
     *
     *  timestamp: ..., // timestamp from parameter
     *
     *  tags: ..., // combined tags from this reporter and the metric
     *
     *  values..., // metric specific values
     *
     *  type..., // metric type
     *
     * }
     *
     * @static
     * @returns {MetricDocumentBuilder}
     * @memberof ElasticsearchMetricReporter
     */
    static defaultDocumentBuilder(): MetricDocumentBuilder;
    /**
     * Gets the values for the specified monotone counter metric.
     *
     * @static
     * @param {MonotoneCounter} counter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getMonotoneCounterValues(counter: MonotoneCounter): {};
    /**
     * Gets the values for the specified counter metric.
     *
     * @static
     * @param {Counter} counter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getCounterValues(counter: Counter): {};
    /**
     * Gets the values for the specified {Gauge} metric.
     *
     * @static
     * @param {Gauge<any>} gauge
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getGaugeValue(gauge: Gauge<any>): {};
    /**
     * Gets the values for the specified {Histogram} metric.
     *
     * @static
     * @param {Histogram} histogram
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getHistogramValues(histogram: Histogram): {};
    /**
     * Gets the values for the specified {Meter} metric.
     *
     * @static
     * @param {Meter} meter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getMeterValues(meter: Meter): {};
    /**
     * Gets the values for the specified {Timer} metric.
     *
     * @static
     * @param {Timer} timer
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getTimerValues(timer: Timer): {};
    /**
     * Either gets 0 or the specified value.
     *
     * @private
     * @param {number} value
     * @returns {number}
     * @memberof ElasticsearchMetricReporter
     */
    private static getNumber;
    /**
     * Metadata for the logger.
     *
     * @private
     * @type {*}
     * @memberof ElasticsearchMetricReporter
     */
    private readonly logMetadata;
    /**
     * Elasticsearch client used to do reporting.
     *
     * @private
     * @type {Client}
     * @memberof ElasticsearchMetricReporter
     */
    private readonly client;
    /**
     * Creates an instance of ElasticsearchMetricReporter.
     *
     * @param {string} [reporterType] the type of the reporter implementation - for internal use
     */
    constructor({ clientOptions, metricDocumentBuilder, indexnameDeterminator, typeDeterminator, log, reportInterval, unit, clock, scheduler, minReportingTimeout, tags, clusterOptions }: ElasticsearchMetricReporterOption, reporterType?: string);
    /**
     * Gets the logger instance.
     *
     * @returns {Logger}
     * @memberof ElasticsearchMetricReporter
     */
    getLog(): Logger;
    /**
     * Sets the logger instance.
     *
     * @param {Logger} log
     * @memberof ElasticsearchMetricReporter
     */
    setLog(log: Logger): void;
    /**
     * Reports an {@link Event}.
     *
     * @param {TEvent} event
     * @returns {Promise<TEvent>}
     * @memberof ElasticsearchMetricReporter
     */
    reportEvent<TEventData, TEvent extends Event<TEventData>>(event: TEvent): Promise<TEvent>;
    /**
     * Does nothing
     *
     * @returns {Promise<void>}
     * @memberof ElasticsearchMetricReporter
     */
    flushEvents(): Promise<void>;
    /**
     * Send the combinations of index and document to the elasticsearch cluster
     * using the bulk method of the elasticsearch client.
     *
     * @protected
     * @param {MetricRegistry | null} registry
     * @param {Date} date
     * @param {MetricType} type
     * @param {Array<ReportingResult<any, any[]>>} results
     * @returns {Promise<void>}
     * @memberof ElasticsearchMetricReporter
     */
    protected handleResults(ctx: OverallReportContext, registry: MetricRegistry | null, date: Date, type: MetricType, results: Array<ReportingResult<any, any[]>>): Promise<void>;
    /**
     * Generalized reporting method of all types of metric instances.
     * Builds the index configuration document and the metric document.
     *
     * @protected
     * @param {Metric} metric
     * @param {ReportingContext<Metric>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportMetric(metric: Metric, ctx: MetricSetReportContext<Metric>): Array<{}>;
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {(MonotoneCounter | Counter)} counter
     * @param {(ReportingContext<MonotoneCounter | Counter>)} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportCounter(counter: MonotoneCounter | Counter, ctx: MetricSetReportContext<MonotoneCounter | Counter>): Array<{}>;
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Gauge<any>} gauge
     * @param {ReportingContext<Gauge<any>>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportGauge(gauge: Gauge<any>, ctx: MetricSetReportContext<Gauge<any>>): Array<{}>;
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Histogram} histogram
     * @param {ReportingContext<Histogram>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportHistogram(histogram: Histogram, ctx: MetricSetReportContext<Histogram>): Array<{}>;
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Meter} meter
     * @param {ReportingContext<Meter>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportMeter(meter: Meter, ctx: MetricSetReportContext<Meter>): Array<{}>;
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Timer} timer
     * @param {ReportingContext<Timer>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    protected reportTimer(timer: Timer, ctx: MetricSetReportContext<Timer>): Array<{}>;
}
