"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElasticsearchMetricReporter = void 0;
require("source-map-support/register");
const elasticsearch_1 = require("@elastic/elasticsearch");
const inspector_metrics_1 = require("inspector-metrics");
/**
 * A MetricReporter extension used to publish metric values to elasticsearch.
 *
 * @export
 * @class ElasticsearchMetricReporter
 * @extends {MetricReporter}
 */
class ElasticsearchMetricReporter extends inspector_metrics_1.ScheduledMetricReporter {
    /**
     * Creates an instance of ElasticsearchMetricReporter.
     *
     * @param {string} [reporterType] the type of the reporter implementation - for internal use
     */
    constructor({ clientOptions, metricDocumentBuilder = ElasticsearchMetricReporter.defaultDocumentBuilder(), indexnameDeterminator = ElasticsearchMetricReporter.dailyIndex('metric'), typeDeterminator = ElasticsearchMetricReporter.defaultTypeDeterminator(), log = console, reportInterval = 1000, unit = inspector_metrics_1.MILLISECOND, clock = new inspector_metrics_1.StdClock(), scheduler = setInterval, minReportingTimeout = 1, tags = new Map(), clusterOptions = new inspector_metrics_1.DefaultClusterOptions() }, reporterType) {
        super({
            clientOptions,
            clock,
            clusterOptions,
            indexnameDeterminator,
            log,
            metricDocumentBuilder,
            minReportingTimeout,
            reportInterval,
            scheduler,
            tags,
            typeDeterminator,
            unit
        }, reporterType);
        this.logMetadata = {
            reportInterval,
            tags,
            unit
        };
        this.client = new elasticsearch_1.Client(clientOptions);
    }
    /**
     * Returns a {@link MetricInfoDeterminator} that returns 'metric' as type.
     *
     * @static
     * @returns {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporter
     */
    static defaultTypeDeterminator() {
        return (registry, metric, type, date) => 'metric';
    }
    /**
     * Returns a {@link MetricInfoDeterminator} that returns an indexname like '<baseName>-yyyy-mm-dd'.
     *
     * @static
     * @param {string} baseName The
     * @returns {MetricInfoDeterminator}
     * @memberof ElasticsearchMetricReporter
     */
    static dailyIndex(baseName) {
        return (registry, metric, type, date) => {
            const day = date.getDate();
            const dayPrefix = (day >= 10) ? '' : '0';
            const month = date.getMonth() + 1;
            const monthPrefix = (month >= 10) ? '' : '0';
            return `${baseName}-${date.getFullYear()}-${monthPrefix}${month}-${dayPrefix}${day}`;
        };
    }
    /**
     * Returns a {@link MetricDocumentBuilder} that builds an object for a metric like this:
     *
     * {
     *
     *  name: ..., // name of metric
     *
     *  group: ..., // group of metric
     *
     *  timestamp: ..., // timestamp from parameter
     *
     *  tags: ..., // combined tags from this reporter and the metric
     *
     *  values..., // metric specific values
     *
     *  type..., // metric type
     *
     * }
     *
     * @static
     * @returns {MetricDocumentBuilder}
     * @memberof ElasticsearchMetricReporter
     */
    static defaultDocumentBuilder() {
        return (registry, metric, type, timestamp, tags) => {
            let values = null;
            if (metric instanceof inspector_metrics_1.MonotoneCounter) {
                values = ElasticsearchMetricReporter.getMonotoneCounterValues(metric);
            }
            else if (metric instanceof inspector_metrics_1.Counter) {
                values = ElasticsearchMetricReporter.getCounterValues(metric);
            }
            else if (metric instanceof inspector_metrics_1.Histogram) {
                values = ElasticsearchMetricReporter.getHistogramValues(metric);
            }
            else if (metric instanceof inspector_metrics_1.Meter) {
                values = ElasticsearchMetricReporter.getMeterValues(metric);
            }
            else if (metric instanceof inspector_metrics_1.Timer) {
                values = ElasticsearchMetricReporter.getTimerValues(metric);
            }
            else {
                values = ElasticsearchMetricReporter.getGaugeValue(metric);
            }
            if (values === null) {
                return null;
            }
            const name = metric.getName();
            const group = metric.getGroup();
            return { name, group, tags, timestamp, values, type };
        };
    }
    /**
     * Gets the values for the specified monotone counter metric.
     *
     * @static
     * @param {MonotoneCounter} counter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getMonotoneCounterValues(counter) {
        const count = counter.getCount();
        if (!count || isNaN(count)) {
            return null;
        }
        return { count };
    }
    /**
     * Gets the values for the specified counter metric.
     *
     * @static
     * @param {Counter} counter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getCounterValues(counter) {
        const count = counter.getCount();
        if (!count || isNaN(count)) {
            return null;
        }
        return { count };
    }
    /**
     * Gets the values for the specified {Gauge} metric.
     *
     * @static
     * @param {Gauge<any>} gauge
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getGaugeValue(gauge) {
        const value = gauge.getValue();
        if ((!value && value !== 0) || Number.isNaN(value)) {
            return null;
        }
        if (typeof value === 'object') {
            return value;
        }
        return { value };
    }
    /**
     * Gets the values for the specified {Histogram} metric.
     *
     * @static
     * @param {Histogram} histogram
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getHistogramValues(histogram) {
        const value = histogram.getCount();
        if (!value || isNaN(value)) {
            return null;
        }
        const snapshot = histogram.getSnapshot();
        const values = {};
        values.count = value;
        values.max = this.getNumber(snapshot.getMax());
        values.mean = this.getNumber(snapshot.getMean());
        values.min = this.getNumber(snapshot.getMin());
        values.p50 = this.getNumber(snapshot.getMedian());
        values.p75 = this.getNumber(snapshot.get75thPercentile());
        values.p95 = this.getNumber(snapshot.get95thPercentile());
        values.p98 = this.getNumber(snapshot.get98thPercentile());
        values.p99 = this.getNumber(snapshot.get99thPercentile());
        values.p999 = this.getNumber(snapshot.get999thPercentile());
        values.stddev = this.getNumber(snapshot.getStdDev());
        return values;
    }
    /**
     * Gets the values for the specified {Meter} metric.
     *
     * @static
     * @param {Meter} meter
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getMeterValues(meter) {
        const value = meter.getCount();
        if (!value || isNaN(value)) {
            return null;
        }
        const values = {};
        values.count = value;
        values.m15_rate = this.getNumber(meter.get15MinuteRate());
        values.m1_rate = this.getNumber(meter.get1MinuteRate());
        values.m5_rate = this.getNumber(meter.get5MinuteRate());
        values.mean_rate = this.getNumber(meter.getMeanRate());
        return values;
    }
    /**
     * Gets the values for the specified {Timer} metric.
     *
     * @static
     * @param {Timer} timer
     * @returns {{}}
     * @memberof ElasticsearchMetricReporter
     */
    static getTimerValues(timer) {
        const value = timer.getCount();
        if (!value || isNaN(value)) {
            return null;
        }
        const snapshot = timer.getSnapshot();
        const values = {};
        values.count = value;
        values.m15_rate = this.getNumber(timer.get15MinuteRate());
        values.m1_rate = this.getNumber(timer.get1MinuteRate());
        values.m5_rate = this.getNumber(timer.get5MinuteRate());
        values.max = this.getNumber(snapshot.getMax());
        values.mean = this.getNumber(snapshot.getMean());
        values.mean_rate = this.getNumber(timer.getMeanRate());
        values.min = this.getNumber(snapshot.getMin());
        values.p50 = this.getNumber(snapshot.getMedian());
        values.p75 = this.getNumber(snapshot.get75thPercentile());
        values.p95 = this.getNumber(snapshot.get95thPercentile());
        values.p98 = this.getNumber(snapshot.get98thPercentile());
        values.p99 = this.getNumber(snapshot.get99thPercentile());
        values.p999 = this.getNumber(snapshot.get999thPercentile());
        values.stddev = this.getNumber(snapshot.getStdDev());
        return values;
    }
    /**
     * Either gets 0 or the specified value.
     *
     * @private
     * @param {number} value
     * @returns {number}
     * @memberof ElasticsearchMetricReporter
     */
    static getNumber(value) {
        if (isNaN(value)) {
            return 0;
        }
        return value;
    }
    /**
     * Gets the logger instance.
     *
     * @returns {Logger}
     * @memberof ElasticsearchMetricReporter
     */
    getLog() {
        return this.options.log;
    }
    /**
     * Sets the logger instance.
     *
     * @param {Logger} log
     * @memberof ElasticsearchMetricReporter
     */
    setLog(log) {
        this.options.log = log;
    }
    /**
     * Reports an {@link Event}.
     *
     * @param {TEvent} event
     * @returns {Promise<TEvent>}
     * @memberof ElasticsearchMetricReporter
     */
    reportEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = this.reportGauge(event, {
                date: event.getTime(),
                metrics: [],
                overallCtx: null,
                registry: null,
                type: 'gauge'
            });
            if (result) {
                yield this.handleResults(null, null, event.getTime(), 'gauge', [{
                        metric: event,
                        result
                    }]);
            }
            return event;
        });
    }
    /**
     * Does nothing
     *
     * @returns {Promise<void>}
     * @memberof ElasticsearchMetricReporter
     */
    flushEvents() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    /**
     * Send the combinations of index and document to the elasticsearch cluster
     * using the bulk method of the elasticsearch client.
     *
     * @protected
     * @param {MetricRegistry | null} registry
     * @param {Date} date
     * @param {MetricType} type
     * @param {Array<ReportingResult<any, any[]>>} results
     * @returns {Promise<void>}
     * @memberof ElasticsearchMetricReporter
     */
    handleResults(ctx, registry, date, type, results) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = results
                .map((result) => result.result)
                .reduce((p, c) => p.concat(c), []);
            if (!body || body.length === 0) {
                return yield Promise.resolve();
            }
            try {
                const response = yield this.client.bulk({ body });
                if (this.options.log) {
                    const warnings = response.warnings;
                    this.options.log.debug(
                    // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
                    `wrote ${type} metrics - warnings ${warnings}`, this.logMetadata);
                }
            }
            catch (reason) {
                if (this.options.log) {
                    const message = reason.message;
                    this.options.log
                        .error(`error writing ${type} metrics - reason: ${message}`, reason, this.logMetadata);
                }
            }
        });
    }
    /**
     * Generalized reporting method of all types of metric instances.
     * Builds the index configuration document and the metric document.
     *
     * @protected
     * @param {Metric} metric
     * @param {ReportingContext<Metric>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportMetric(metric, ctx) {
        const document = this.options.metricDocumentBuilder(ctx.registry, metric, ctx.type, ctx.date, this.buildTags(ctx.registry, metric));
        if (document) {
            const _index = this.options.indexnameDeterminator(ctx.registry, metric, ctx.type, ctx.date);
            const _type = this.options.typeDeterminator(ctx.registry, metric, ctx.type, ctx.date);
            return [
                { index: { _index, _type } },
                document
            ];
        }
        return [];
    }
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {(MonotoneCounter | Counter)} counter
     * @param {(ReportingContext<MonotoneCounter | Counter>)} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportCounter(counter, ctx) {
        return this.reportMetric(counter, ctx);
    }
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Gauge<any>} gauge
     * @param {ReportingContext<Gauge<any>>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportGauge(gauge, ctx) {
        return this.reportMetric(gauge, ctx);
    }
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Histogram} histogram
     * @param {ReportingContext<Histogram>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportHistogram(histogram, ctx) {
        return this.reportMetric(histogram, ctx);
    }
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Meter} meter
     * @param {ReportingContext<Meter>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportMeter(meter, ctx) {
        return this.reportMetric(meter, ctx);
    }
    /**
     * Calls {@link #reportMetric} with the specified arguments.
     *
     * @protected
     * @param {Timer} timer
     * @param {ReportingContext<Timer>} ctx
     * @returns {Array<{}>}
     * @memberof ElasticsearchMetricReporter
     */
    reportTimer(timer, ctx) {
        return this.reportMetric(timer, ctx);
    }
}
exports.ElasticsearchMetricReporter = ElasticsearchMetricReporter;
//# sourceMappingURL=ElasticsearchMetricReporter.js.map